package org.netkernel.module.standard.builtin.branchmerge;

import org.netkernel.container.IKernel;
import org.netkernel.module.standard.ILateBound;
import org.netkernel.module.standard.StandardSpace;
import org.netkernel.module.standard.StandardSpaceMeta;
import org.netkernel.module.standard.endpoint.IStandardEndpoint;
import org.netkernel.layer0.meta.ISpaceElements;
import org.netkernel.layer0.meta.ISpaceMeta;
import org.netkernel.layer0.meta.impl.SpaceElementsImpl;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResolutionContext;
import org.netkernel.layer0.nkf.impl.NKFLifecycleEventContextImpl;
import org.netkernel.layer0.nkf.impl.ParametersImpl;
import org.netkernel.layer0.util.TupleList;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.ClassLoaderWithExports;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.ISpace;

public class BranchMergeSpace extends StandardSpace
{
	private IStandardEndpoint mOverlay;
	private ISpace mDelegateSpace;
	private ISpace mNextSpace;
	private BranchMergeOverlay mOwner;
	
	public BranchMergeSpace(BranchMergeOverlay aBM, IStandardEndpoint aOverlay, ISpace aDelegateSpace, ISpace aNextSpace, String aName, BranchMergeOverlay aOwner)
	{	super(aBM.getStandardSpace(),aBM.getStandardSpace().getOwningModule(),null);
		setName(aName);
		setMappings(new IStandardEndpoint[]{aOverlay});
		mOverlay=aOverlay;
		mDelegateSpace=aDelegateSpace;
		mNextSpace=aNextSpace;
		mOwner=aOwner;
	}
	
	public void postCommissionSpace() throws Exception
	{	super.postCommissionSpace();
	
		if (mOverlay instanceof ILateBound)
		{	ILateBound lb = (ILateBound)mOverlay;
			IKernel k=getKernel();
			IRequestScopeLevel scope=RequestScopeLevelImpl.createRootScopeLevel(this);
			NKFLifecycleEventContextImpl context = new NKFLifecycleEventContextImpl(k,scope,this,ParametersImpl.EMPTY);
			mOverlay=lb.bind(context);
			mOverlay.onCommissionEndpoint(k, this);
		}
	
		IRequestScopeLevel scope = RequestScopeLevelImpl.createRootScopeLevel(this);
		mOverlay.postCommissionMapping(scope);
	}
	
	
	
	public void preDecommissionSpace() throws Exception
	{	IRequestScopeLevel scope = RequestScopeLevelImpl.createRootScopeLevel(this);
		mOverlay.preDecommissionMapping(scope);
		super.preDecommissionSpace();
	}
	
	public void onResolve(INKFResolutionContext aContext) throws Exception
	{	
		INKFRequestReadOnly req=aContext.getRequestToResolve();
		if (req.getVerb()==INKFRequestReadOnly.VERB_META)
		{	if (req.getIdentifier().equals(""))
			{	aContext.createResolutionResponse(this);
			}
			else
			{	aContext.delegateResolution(mDelegateSpace);
			}
		}
		else
		{	mOverlay.onResolve(aContext);
			if (!aContext.isResponseSet())
			{	boolean ready=!(mOverlay instanceof ILateBound) || ((ILateBound)mOverlay).isBound();
				if (!ready)
				{	aContext.delegateResolution(mDelegateSpace);
				}
			}
		}
	}

	public void onMeta(INKFRequestContext aContext) throws Exception
	{	StandardSpaceMeta result = new StandardSpaceMeta(getName(),null,null,null);
	
		ISpaceMeta delegateMeta=mOwner.getDelegateMeta(aContext);
		ISpaceElements se=delegateMeta.getElements();
		TupleList partsTuples = new TupleList(3,32);
		for (int i=0; i<se.size(); i++)
		{	IIdentifier id=se.getIdentifier(i);
			partsTuples.put(id,mNextSpace,mOverlay);
		}
		
		
		result.setParts(new SpaceElementsImpl(partsTuples));
		aContext.createResponseFrom(result);
	}
	
	public ISpace getDelegateSpace()
	{	return mDelegateSpace;
	}
	
	public ISpace getNextSpace()
	{	return mNextSpace;
	}
	
	public String toString()
	{	return "BranchMergeSpace["+getName()+"]";
	}
	
	public ClassLoaderWithExports getClassLoader()
	{	return mDelegateSpace.getClassLoader();
	} 
}